@@ -43,12 +43,12 @@ using namespace KScreen;
 
 WaylandConfig::WaylandConfig(QObject *parent)
     : QObject(parent)
-    , m_screen(new WaylandScreen(this))
     , m_outputManagement(nullptr)
-    , m_blockSignals(true)
     , m_registryInitialized(false)
+    , m_blockSignals(true)
     , m_newOutputId(0)
     , m_kscreenConfig(nullptr)
+    , m_screen(new WaylandScreen(this))
 {
     connect(this, &WaylandConfig::initialized, &m_syncLoop, &QEventLoop::quit);
     QTimer::singleShot(1000, this, [this] {
@@ -236,6 +236,8 @@ void WaylandConfig::removeOutput(quint32 name)
 
 void WaylandConfig::updateKScreenConfig(KScreen::ConfigPtr &config) const
 {
+    auto features = Config::Feature::Writable | Config::Feature::PerOutputScaling;
+    config->setSupportedFeatures(features);
     config->setValid(m_connection->display());
     KScreen::ScreenPtr screen = config->screen();
     m_screen->updateKScreenScreen(screen);
@@ -295,6 +297,10 @@ void WaylandConfig::applyConfig(const KScreen::ConfigPtr &newConfig)
             wlOutputConfiguration->setPosition(o_old->outputDevice(), output->pos());
         }
 
+        if (device->scale() != output->scale()) {
+            wlOutputConfiguration->setScale(o_old->outputDevice(), output->scale());
+        }
+
         // rotation
         auto r_current = o_old->toKScreenRotation(device->transform());
         auto r_new = output->rotation();
@@ -309,8 +315,6 @@ void WaylandConfig::applyConfig(const KScreen::ConfigPtr &newConfig)
         if (w_newmodeid != w_currentmodeid) {
             wlOutputConfiguration->setMode(device, w_newmodeid);
         }
-
-        // FIXME: scale
     }
 
     // We now block changes in order to compress events while the compositor is doing its thing
@@ -329,4 +333,4 @@ void WaylandConfig::applyConfig(const KScreen::ConfigPtr &newConfig)
     blockSignals();
     // Now ask the compositor to apply the changes
     wlOutputConfiguration->apply();
-}
\ No newline at end of file
+}
